part of tecfy_core_client_package;

class EntityListController extends BaseController {
  EntityService _service;
  Entity entity;
  List<dynamic> data;
  List<dynamic> filterdData;
  List<EntityFilter> filters;
  List<Property> searchableProperties;
  int pageIndex = 0;
  int pageIndexMax = 0;
  bool hasNextPage = true;
  
  TextEditingController searchController;
  RefreshController refreshController =
      RefreshController(initialRefresh: false); // for pull to refrsh

  EntityListController(State<StatefulWidget> state, String entityName)
      : super(state) {
    refreshController.footerMode.addListener(footerListner);
    searchController = TextEditingController();
    _service = EntityService();
    filters = List<EntityFilter>();
    print('-------------- EntityListController => $entityName');
    entity = EntityService.getEntity(entityName);
    searchableProperties = entity.properties.where((e) => e.isSearchable).toList();
    loadData();
  }

  @override
  dispose() {
    refreshController?.dispose();
    searchController?.dispose();
    super.dispose();
  }

  String sortProp;
  bool sortAsc = true;

  set globalSearchText(String value){
    if(value == null || value.isEmpty)
    {
      filterdData = null;
    }else{
      filterdData = data.where((d){
        return searchableProperties.any((p)=> d[p.dbName].toString().contains(value));
      }).toList();
    }
    setState();
  }

  sortBy(Property prop) {
    if (prop.dbName != sortProp) {
      sortProp = prop.dbName;
      sortAsc = true;
    } else
      sortAsc = !sortAsc;

    data.sort((a, b) => sortAsc
        ? a[sortProp].toString().compareTo(b[sortProp].toString())
        : b[sortProp].toString().compareTo(a[sortProp].toString()));
    setState();
  }

  bool get hasPageing => (data?.length ?? 0) >= EntityService.pageSize;

  footerListner() async {
    print('==========on load ====== ${refreshController.footerStatus}');
    if (refreshController.footerStatus != LoadStatus.loading) return;
    var success = await gotoMextPageIndex();
    if (success && hasNextPage) {
      refreshController.loadComplete();
    } else if (success && !hasNextPage) {
      //refreshController.loadComplete();
      refreshController.loadNoData();
    } else {
      refreshController.loadFailed();
    }
  }

  Future<bool> gotoMextPageIndex() {
    pageIndex++;
    pageIndexMax = max(pageIndexMax, pageIndex);
    return loadData(loadFromServer: true, withLoading: false);
  }

  gotoPageIndex(int i) {
    pageIndex = i;
    setLoading();
    loadData(loadFromServer: true);
  }

  var isLoading = false;
  Future<bool> loadData(
      {bool loadFromServer = false, bool withLoading = true}) async {
    if (isLoading) return true;
    isLoading = true;
    var result = await _service.getData(entity,
        filter: filters, forceFromServer: loadFromServer, pageIndex: pageIndex);
    if (withLoading) setResume();
    if (!result.success) {
      data = [];
      if (withLoading) {
        setState();
        result.showMessage(context);
      }
      isLoading = false;
      return false;
    } else {
      if (LayoutService.isMobile(context) && pageIndex > 0)
        data.addAll(result.data);
      else
        data = result.data;

      if (result.data.length < EntityService.pageSize) hasNextPage = false;
    }
    setState();
    // if(pageIndex == 0) refreshAnimation();
    isLoading = false;
    return true;
  }

  Future loadFromServer() async {
    await loadData(loadFromServer: true);
    refreshController.refreshCompleted();
  }

  Future onRefresh() async {
    pageIndex = 0;
    pageIndexMax = 0;
    hasNextPage = true;
    setLoading();
    await loadData(loadFromServer: true);
  }

  Future onAdd() async {
    var result = await AppNavigator.navigateTo(
        context, () => EntityFormPage(entity.dbName),
        animationType: LayoutService.isMobile(context)
            ? AnimationTypes.slideUp
            : AnimationTypes.opacity);
    if (result == true) {
      loadFromServer();
    }
  }

  Future onUpdate(d) async {
    var result = await AppNavigator.navigateTo(
        context, () => EntityFormPage(entity.dbName, d),
        animationType: LayoutService.isMobile(context)
            ? AnimationTypes.slideUp
            : AnimationTypes.opacity);
    if (result == true) {
      loadFromServer();
    }
  }

  Future onDelete(d) async {
    if (await confirmMessage('هل انت متاكد من الحذف؟')) {
      var result = await EntityService().delete(entity, d['id']);
      if (result.success) {
        showMessage('تم الحذف بنجاح', type: MessageType.info);
        loadFromServer();
      } else {
        showMessage('خطأ اثناء الحذف', type: MessageType.error);
      }
    }
  }

  lowerFirst(String s) {
    return '${s[0].toLowerCase()}${s.substring(1)}';
  }

  String get getPhotoDataPropName =>
      '${entity.properties.firstWhere((d) => d.isImage).dbName}Id';
}
