part of tecfy_core_client_package;

class EntityListPage extends StatefulWidget {
  final String entity;
  EntityListPage(this.entity);
  @override
  _EntityListPageState createState() => _EntityListPageState();
}

class _EntityListPageState extends AppState<EntityListPage> {
  EntityListController _controller;

  @override
  void initState() {
    _controller = EntityListController(this, widget.entity);
    super.initState();
  }

  _onActionClicked(EntityAction action, Map data) async {
    Map form = Map<String, dynamic>();
    await AppNavigator.showAppDialog(
        context,
        (dialogContext) => Container(
            width: 300,
            height: 500,
            child: Card(
              child: Column(
                children: <Widget>[
                  Chip(label: Text(action.name)),
                  Column(
                    children: action.actionFormProp
                        .map((p) => PorpertyComponent(
                            property: p, onChange: (v) => form[p.dbName] = v))
                        .toList(),
                  ),
                  AppButtonPost(
                      label: action.name,
                      onPressed: () async {
                        Navigator.pop(dialogContext);
                      })
                ],
              ),
            )));
    // print(result);
    // print(form);
    for (var prop in action.actionFormProp) {
      if (prop.isRequired && form[prop.dbName] == null) {
        AppNavigator.showMessage(context, 'Property required : ${prop.name}');
        return;
      }
    }
    _controller.setLoading();
    form['id'] = data['id'];
    var postResult = await ApiService.httpPost(
        'Entity/${_controller.entity.dbName}/action/${action.dbName}',
        form,
        true);
    _controller.setResume();
    _controller.showMessage(postResult.success ? 'Done' : 'Unknown Error!');
  }

  _onOpenFilter() async {
    var result = await AppNavigator.navigateTo(
        context,
        () => EntityFilterPage(_controller.entity.name,
            _controller.entity.properties, _controller.filters),
        animationType: AnimationTypes.slideLeft);
    bool canceled = false;
    if (result == null) canceled = true;
    _controller.filters = (result as List<EntityFilter>)
        .where((f) =>
            f.value != null && f.property != null && f.operatorId != null)
        .toList();

    if (!canceled) {
      _controller.setLoading();
      _controller.loadData();
    }
  }

  @override
  Widget build(BuildContext context) {
    return BasePage(context, _controller.entity.name,
        appBarActions: LayoutService.isMobile(context)
            ? [
                IconButton(
                    icon: Icon(Icons.filter_list,
                        color: _controller.filters.length > 0
                            ? Colors.redAccent
                            : ThemeService.mainColor),
                    onPressed: _onOpenFilter)
              ]
            : null,
        floatingActionButtonWidget: LayoutService.isMobile(context) &&
                _controller.entity.permission.canAdd
            ? FloatingActionButton(
                backgroundColor: ThemeService.mainColor,
                onPressed: _controller.onAdd,
                child: Icon(Icons.add))
            : null,
        child: _controller.data == null
            ? Center(
                child: CircularProgressIndicator(
                    valueColor:
                        AlwaysStoppedAnimation<Color>(ThemeService.mainColor)))
            : LayoutService.bindWidget(context,
                mobileWidget: _getMobileWidget(),
                descktopWidget: _getDescktopWidget()));
  }

  Map<int, TableColumnWidth> getColumnWidths() {
    var columns = _controller.entity.properties
        .where((e) =>
            e.isGrid &&
            (e.language == Language.notSelected ||
                e.language == LocalConfigService.language))
        .toList();

    Map<int, TableColumnWidth> data = {};
    int testCount = 0;
    for (int i = 0; i < columns.length; i++) {
      switch (columns[i].propertyType) {
        case PropertyTypes.Text:
          data[i] = FlexColumnWidth(testCount == 1 ? 2 : 1);
          testCount++;
          break;
        case PropertyTypes.File:
          data[i] = FixedColumnWidth(60);
          break;
        case PropertyTypes.Number:
          data[i] = FlexColumnWidth(0.5);
          break;
        case PropertyTypes.Boolean:
          data[i] = FixedColumnWidth(40);
          break;
        default:
          data[i] = FlexColumnWidth(1);
          break;
      }
    }
    data[columns.length] = FixedColumnWidth(40);
    data[columns.length + 1] = FixedColumnWidth(40);
    return data;
  }

  Widget _getDescktopWidget() {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.stretch,
      children: <Widget>[
        Row(children: <Widget>[
          Expanded(
              child: SearchBox(
                  searchTextController: _controller.searchController,
                  placeholder: trans('Search'),
                  onChange: (v) => _controller.globalSearchText = v)),
          AppButtonIconText(
              onPressed: _controller.onAdd,
              label: trans('Add'),
              height: 35,
              icon: Icon(Icons.add, color: ThemeService.textOnMainColor),
              color: ThemeService.secondryColor),
          SizedBox(width: 10),
          AppButtonIconText(
              onPressed: _controller.onRefresh,
              label: trans('Refresh'),
              height: 35,
              icon: Icon(Icons.refresh, color: ThemeService.textOnMainColor),
              color: ThemeService.subColor),
          SizedBox(width: 10),
        ]),
        Expanded(
            child: Scrollbar(
                child: SingleChildScrollView(
          child: Container(
              margin: EdgeInsets.symmetric(horizontal: 30, vertical: 5),
              child: AnimationLimiter(
                //key:UniqueKey(),
                child: Table(
                  columnWidths: getColumnWidths(),
                  children: _getTableRows(),
                ),
              )),
        ))),
        _controller.hasPageing
            ? SizedBox(
                height: 50,
                child: ListView.builder(
                  itemCount: (_controller.pageIndexMax + 2),
                  scrollDirection: Axis.horizontal,
                  itemBuilder: (_context, pageIndex) => pageIndex ==
                          _controller.pageIndexMax + 1
                      ? (_controller.hasNextPage
                          ? FlatButton(
                              child: Text(
                                  AppLocale.translate(context, 'Next Page')),
                              onPressed: _controller.gotoMextPageIndex)
                          : Container())
                      : IconButton(
                          padding: EdgeInsets.all(0),
                          icon: Text(
                            (pageIndex + 1).toString(),
                            style: TextStyle(
                                color: pageIndex == _controller.pageIndex
                                    ? ThemeService.mainColor
                                    : ThemeService.secondryColor),
                          ),
                          onPressed: () => _controller.gotoPageIndex(pageIndex),
                        ),
                ),
              )
            : Container()
      ],
    );
  }

  List<TableRow> _getTableRows() {
    int rowIndex = 0;
    return (_controller.filterdData ?? _controller.data)
        .map<TableRow>((d) => _getTableRow(d, rowIndex++))
        .toList()
          ..insert(0, _getHeaderRow());
  }

  TableRow _getTableRow(d, rowIndex) {
    return TableRow(
      decoration: BoxDecoration(
          color: rowIndex % 2 == 0
              ? ThemeService.backgroundSecondColor
              : ThemeService.backgroundColor),
      children: AnimationConfiguration.toStaggeredList(
        duration: const Duration(milliseconds: 375),
        childAnimationBuilder: (widget) => SlideAnimation(
            horizontalOffset: 50.0,
            child: SlideAnimation(
              verticalOffset: 50.0,
              child: FadeInAnimation(
                child: widget,
              ),
            )),
        children: _controller.entity.properties
            .where((e) =>
                e.isGrid &&
                (e.language == Language.notSelected ||
                    e.language == LocalConfigService.language))
            .map<Widget>((p) => _getCell(p, d))
            .toList()
              ..add(_editCell(d))
              ..add(_deleteCell(d)),
      ),
    );
  }

  TableRow _getHeaderRow() {
    return TableRow(
      children: _controller.entity.properties
          .where((e) =>
              e.isGrid &&
              (e.language == Language.notSelected ||
                  e.language == LocalConfigService.language))
          .map<TableCell>((p) => _getHeader(p))
          .toList()
            ..add(_getHeaderCell(
                AppLocale.translate(context, 'Edit'), false, null, true))
            ..add(_getHeaderCell(
                AppLocale.translate(context, 'Delete'), false, null, true)),
    );
  }

  Widget _editCell(Map data) {
    return IconButton(
      tooltip: trans('Edit'),
      padding: EdgeInsets.all(0),
      icon: Icon(Icons.note, color: ThemeService.mainColor),
      onPressed: () => _controller.onUpdate(data),
    );
  }

  Widget _deleteCell(Map data) {
    return data['isLocked'] == true
        ? Container()
        : IconButton(
            tooltip: trans('Delete'),
            padding: EdgeInsets.all(0),
            icon: Icon(Icons.delete, color: ThemeService.dangerColor),
            onPressed: () => _controller.onDelete(data),
          );
  }

  TableCell _getHeader(Property prop) {
    return _getHeaderCell(prop.name, prop.dbName == _controller.sortProp,
        () => _controller.sortBy(prop));
  }

  TableCell _getHeaderCell(String text,
      [bool isSelected = false, onPressed, bool isCenter = false]) {
    return TableCell(
        child: MaterialButton(
            padding: EdgeInsets.all(0),
            child: Align(
                alignment: isCenter
                    ? Alignment.center
                    : LocalConfigService.isArabic
                        ? Alignment.centerRight
                        : Alignment.centerLeft,
                child: Text('$text ${isSelected ? '*' : ''}',
                    textAlign: TextAlign.start,
                    style: TextStyle(
                        fontSize: 12,
                        fontWeight:
                            isSelected ? FontWeight.bold : FontWeight.normal,
                        color: ThemeService.secondryColor
                        //color: isSelected ? ThemeService.mainColor : ThemeService.secondryColor
                        ))),
            onPressed: onPressed));
  }

  Widget _getCell(Property prop, data, {totals: false}) {
    if (prop.isImage) {
      if (data[prop.dbName + 'Id'] == null ||
          data[prop.dbName + 'Id'].toString().isEmpty) {
        return Container();
      }      
      return Align(
          alignment: LocalConfigService.isArabic
              ? Alignment.centerRight
              : Alignment.centerLeft,
          child: Image(
              image: ImageService.imageProvider(data[prop.dbName + 'Id']),
              height: 50));
    }

    var val = data[prop.dbName];
    if (totals) {
      if (val == null || val == 0 || val == '' || val == '0') {
        return Container();
      }
    } else {
      if (val == null) {
        return Container();
      }
    }

    Widget widgetValue;
    switch (prop.valueType) {
      case ValueTypes.Text:
        widgetValue = Text(
          val.toString(),
          style: totals ? ThemeService.titleStyle : TextStyle(fontSize: 16),
        );
        break;
      case ValueTypes.Number:
        //widgetValue =Text(val.toString(),style: totals? ThemeService.titleStyle :TextStyle(fontSize: 12));
        if (prop.propertyType == PropertyTypes.DateTime)
          widgetValue = Text(
            DateTime.fromMillisecondsSinceEpoch(val, isUtc: true).toString(),
            style: totals ? ThemeService.titleStyle : TextStyle(fontSize: 10),
          );
        else
          widgetValue = prop.getDisplayWidget(data);
        break;
      case ValueTypes.DateTime:
        if (prop.propertyType == PropertyTypes.DateTime)
          widgetValue = Text(
            val.toString().replaceFirst('T', ' ').substring(0, 16),
            style: totals ? ThemeService.titleStyle : TextStyle(fontSize: 10),
          );
        else
          widgetValue = Text(
            val.toString().length > 10
                ? val.toString().substring(0, 10)
                : val.toString(),
            style: totals ? ThemeService.titleStyle : TextStyle(fontSize: 10),
          );
        break;
      case ValueTypes.Boolean:
        widgetValue = Padding(
          padding: EdgeInsets.only(top: 7, right: 2, left: 2),
          child: Align(
            alignment: LocalConfigService.isArabic
                ? Alignment.centerRight
                : Alignment.centerLeft,
            child: prop.getDisplayWidget(data),
          ),
        ); // Text( val.toString(),style: totals? ThemeService.titleStyle :TextStyle(fontSize: 10));
        break;
      default:
        widgetValue = Text('N/A [${prop.valueType}]');
    }

    return Padding(padding: EdgeInsets.only(top: 10), child: widgetValue);
  }

  Widget _getMobileWidget() {
    return Column(children: [
      Padding(
          padding: EdgeInsets.only(right: 10),
          child: SearchBox(
              searchTextController: _controller.searchController,
              placeholder: trans('Search'),
              onChange: (v) => _controller.globalSearchText = v)),
      SizedBox(height: 20),
      Expanded(
          child: AnimationLimiter(
              child: SmartRefresher(
                  enablePullDown: true,
                  enablePullUp: true,
                  onRefresh: _controller.loadFromServer,
                  controller: _controller.refreshController,
                  header: WaterDropHeader(complete: Icon(Icons.done)),
                  footer: CustomFooter(
                    builder: (BuildContext context, LoadStatus mode) {
                      Widget body;
                      if (mode == LoadStatus.idle) {
                        body = Text("pull up load");
                      } else if (mode == LoadStatus.loading) {
                        body = CupertinoActivityIndicator();
                      } else if (mode == LoadStatus.failed) {
                        body = Text("Load Failed!Click retry!");
                      } else if (mode == LoadStatus.canLoading) {
                        body = Text("release to load more");
                      } else {
                        body = Text("No more Data");
                      }
                      return Container(
                        height: 55.0,
                        child: Center(child: body),
                      );
                    },
                  ),
                  child: ListView.builder(
                      itemCount:
                          (_controller.filterdData ?? _controller.data).length,
                      itemBuilder: (BuildContext context, int index) {
                        return AnimationConfiguration.staggeredList(
                            position: index,
                            duration: const Duration(milliseconds: 375),
                            child: SlideAnimation(
                                verticalOffset: 50.0,
                                horizontalOffset: 50.5,
                                child: FlipAnimation(
                                    flipAxis: FlipAxis.y,
                                    child: _getMobileRow(
                                        (_controller.filterdData ??
                                            _controller.data)[index]))));
                      }))
              // ListView(
              //     // controller: _controller.scrollController,
              //     children:
              //         _controller.data.map<Widget>((d) => _getMobileRow(d)).toList()
              //     //..add(_controller.hasNextPage ? FlatButton(child:Text('Load More'),onPressed: _controller.gotoMextPageIndex) : Container())
              //     ),
              ))
    ]);
  }

  Widget _getMobileRow(d) {
    return Slidable(
      // delegate: SlidableDrawerDelegate(),
      actionPane: SlidableDrawerActionPane(),
      actionExtentRatio: 0.25,
      child: Container(
          child: EntityListTileComponent(_controller.entity, d,
              onClick: (d) => _controller.onUpdate(d))),

      actions: _controller.entity.actions
              ?.where((a) => d[a.dbName + 'When'])
              ?.map(
                (a) => IconSlideAction(
                    caption: a.name,
                    color: ThemeService.mainColor,
                    icon: Icons.done,
                    onTap: () =>
                        _onActionClicked(a, d) //_showSnackBar('Archive'),
                    ),
              )
              ?.toList() ??
          [],
      // <Widget>[
      //   IconSlideAction(
      //     caption: 'Archive',
      //     color: Colors.blue,
      //     icon: Icons.archive,
      //     onTap: () => _showSnackBar('Archive'),
      //   ),
      //   IconSlideAction(
      //     caption: 'Share',
      //     color: Colors.indigo,
      //     icon: Icons.share,
      //     onTap: () => _showSnackBar('Share'),
      //   ),
      // ],
      secondaryActions: !_controller.entity.permission.canDelete
          ? []
          : <Widget>[
              // IconSlideAction(
              //   caption: 'More',
              //   color: Colors.black45,
              //   icon: Icons.more_horiz,
              //   onTap: () => _showSnackBar('More'),
              // ),
              IconSlideAction(
                caption: AppLocale.translate(context, 'Delete'),
                color: Colors.red,
                icon: Icons.delete,
                onTap: () => _controller.onDelete(d),
              ),
            ],
    );
  }
}
